
#include <TimedAction.h>
#include "ioConfig.h"
#include "mirror.h"
#include "lcdControl.h"
#include "communication.h"

// void systemRunningLED();

// set pin numbers:
 #include "WProgram.h"
void setup();
void loop();
void systemRunningLED();
void readSensors();
void lcdPolling();
void commTask();
void setupComm();
void activateButton();
void autoCalibrate();
void setupLcdControl();
void lcdTask();
void pressOnOffButton();
void pressMenuButton();
void pressAnsButton();
void pressSrcButton();
void pressVolPlusButton(int num);
void pressVolMinusButton(int num);
void PowerModingTask();
void offState();
void standbyState();
void onState();
void ExecuteDuringStandby();
void ExecuteDuringOn();
int CurrentPowerState();
void notificationOfStatus(int data);
void refreshPowerStatePolling();
void SendNotificationOfStatus(int data);
void checkLightLevel();
void ButtonsTask(void);
void performAction(void);
void ScreenOff(void);
void ScreenOn(void);
void ReqNextProfile(void);
void calibrateLdrButton(void);
const int ledPin =  13;      // the number of the LED pin

 // Variables will change:
 int ledState = LOW;             // ledState used to set the LED
 long previousMillis = 0;        // will store last time LED was updated

 // the follow variables is a long because the time, measured in miliseconds,
 // will quickly become a bigger number than can be stored in an int.
 long interval = 200;           // interval at which to blink (milliseconds)

TimedAction readSensorsAction = TimedAction(25, readSensors);
TimedAction lcdPollingAction = TimedAction(150, lcdPolling);

 void setup()
 {
   setupComm();
   calibrateLdrButton();
   setupLcdControl();
 }


 void loop()
 {
   readSensorsAction.check();
   lcdPollingAction.check();
   PowerModingTask();
   //Testing 
   commTask();
   delay(10);
 }


 void systemRunningLED()
 {
     // if the LED is off turn it on and vice-versa:
     if (ledState == LOW)
       ledState = HIGH;
     else
       ledState = LOW;

     // set the LED with the ledState of the variable:
     digitalWrite(ledPin, ledState);
     //digitalWrite(6, HIGH);
     //delay(100);
     //digitalWrite(7, LOW);
     //delay(100);
 }
 
 void readSensors()
 {
    ButtonsTask();
    systemRunningLED();
 }
 
 void lcdPolling()
 {
   lcdTask();
 }
#include "lcdControl.h"
#include <SimpleMessageSystem.h>

void commTask()
{

  if (messageBuild() > 0)
  { // Checks to see if the message is complete and erases any previous messages
    switch (messageGetChar())
    { // Gets the first word as a character
    case 'b': // Activate the a button from the LCD display
      activateButton(); // Call the readpins function
      break; // Break from the switch
    case 'c': // calibrate
      autoCalibrate();
      break;
    case 'l': // calibrate
      setupLcdControl();
      break;
    case 's': // calibrate
      notificationOfStatus(messageGetInt());
      break;
    }

  }

}

void setupComm()
{
   Serial.begin(9600);

}

void activateButton() { // Write pin
  int data;
  switch (messageGetInt()) { // Gets the next word as a character

    case POWER_BTN : // WRITE an analog pin
       pressOnOffButton();
    break;  // Break from the switch
    case MENU_BTN : // WRITE an analog pin
       pressMenuButton();
    break;  // Break from the switch
    case ANS_BTN : // WRITE an analog pin
       pressAnsButton();
    break;  // Break from the switch
    case SRC_BTN : // WRITE an analog pin
       pressSrcButton();
    break;  // Break from the switch
    case VOL_PLUS_BTN : // WRITE an analog pin
       data = messageGetInt();
       pressVolPlusButton(data);
    break;  // Break from the switch
    case VOL_MINUS_BTN : // WRITE an analog pin
       data = messageGetInt();
       pressVolMinusButton(data);
    break;  // Break from the switch
    default : // WRITE an analog pin
       //nothing
    break;  // Break from the switch
  }
}

void autoCalibrate() {
   calibrateLdrButton();
 }

#include <Pulse.h>
#include "lcdControl.h"
#include "ioConfig.h"

#define pulseWidth      50 //ms



// Set the GPIO used to control the LCD
Pulse powerPulseControl(powerControlPin, 3000);  
Pulse menuPulseControl(menuControlPin, pulseWidth);  
Pulse ansPulseControl(ansControlPin, pulseWidth);  
Pulse srcPulseControl(srcControlPin, pulseWidth);  
Pulse volPlusPulseControl(volPlusControlPin, pulseWidth);  
Pulse volMinusPulseControl(volMinusControlPin, pulseWidth);  

void setupLcdControl()
{
   pressOnOffButton();
}

void lcdTask()
{
   powerPulseControl.polling();
   menuPulseControl.polling();
   ansPulseControl.polling();
   srcPulseControl.polling();
   volPlusPulseControl.polling();
   volMinusPulseControl.polling();
   //Serial.println("Task");
}

void pressOnOffButton()
{
  powerPulseControl.trigger();
}
void pressMenuButton()
{
  menuPulseControl.trigger();
}

void pressAnsButton()
{
  ansPulseControl.trigger();
}
void pressSrcButton()
{
  srcPulseControl.trigger();
}

void pressVolPlusButton(int num)
{
  volPlusPulseControl.trigger(num);
}
void pressVolMinusButton(int num)
{
  volMinusPulseControl.trigger(num);
}

#include "AnalogSwitch.h"
#include "powerModing.h"


// OFF_DELAY (msec)
#define OFF_DELAY   10000


static int currentPowerState = OFF_STATE;
static int newPowerState = STANDBY_STATE;
static int prevPowerState = OFF_STATE;
static unsigned long startTime;
static unsigned long curTime; // this is global variable
unsigned int lightSensorLevel;
boolean goToPrevStateFlag = false;
boolean changeFlag = true;

TimedAction refreshPowerStateAction = TimedAction(750, refreshPowerStatePolling);
AnalogSwitch lightSensor(ldrSensorPort, lightSensorLevel, 0);

 void PowerModingTask()
 {
   switch(currentPowerState)
   {
      case OFF_STATE:
         //Call initialization or any other setup
         offState();
      break;
      case STANDBY_STATE:
         //Call initialization 
         standbyState();
      break;
      case ON_STATE:
         //Call
         standbyState(); 
      break;
   }
   if(newPowerState != currentPowerState)
   {
      currentPowerState = newPowerState;
      //SendNotificationOfStatus(currentPowerState);
   }
   refreshPowerStateAction.check();
 }

 void offState()
 {
   currentPowerState = STANDBY_STATE;
   delay(500);
 }
 
 void standbyState()
 {
     ExecuteDuringStandby();
 }

 void onState()
 {
     ExecuteDuringOn();
 }

 void ExecuteDuringStandby()
 {
   checkLightLevel();
 }
 
 void ExecuteDuringOn()
 {
   //Check the ambient light level
   checkLightLevel();
 
 }
 
 int CurrentPowerState()
 {
    return currentPowerState;
 } 

 void notificationOfStatus(int data)
 {
 }
 
  void refreshPowerStatePolling()
 {
   SendNotificationOfStatus(newPowerState);
 }
 
 void SendNotificationOfStatus(int data)
 {
      messageSendChar('S'); 
      messageSendInt(data);
      messageEnd();
 }
 
 void checkLightLevel()
 {   
      if(lightSensor.Read() && changeFlag)
      {
           #if _DEBUG_MODE_IS
           Serial.println("there is NO light");
           #endif
           prevPowerState = currentPowerState;
           newPowerState = STANDBY_STATE; //Turns off the screen
           goToPrevStateFlag = true;
           changeFlag = false;
      }else
      if(goToPrevStateFlag && !lightSensor.Read())
      {
         #if _DEBUG_MODE_IS
         Serial.println("there is  light");
         #endif
         newPowerState = prevPowerState;
         goToPrevStateFlag = false;
         changeFlag = true;
         SendNotificationOfStatus(newPowerState);
      }
 }




#include "AnalogSwitch.h"
#include "ioConfig.h"

#define LDR_WAIT_TIME 500 //milliseconds 
#define LDR_NEXT_TIME 300 //milliseconds
#define TIME_FOR_INACTIVITY  15000
#define TRESHOLD_WIDTH  10

unsigned int ldrTimeStamp;
unsigned int ldrElapsedTime;
unsigned int lastActionTimeStamp;
unsigned int levelLdrButton;
AnalogSwitch selectBtn(ldrButtonPort, levelLdrButton, TRESHOLD_WIDTH);
boolean selectBtnActive = false;

void ButtonsTask(void)
{
  if(selectBtnActive)
  {
     if(selectBtn.Read())
     {
        if (millis() > (ldrTimeStamp + LDR_WAIT_TIME))
        {
           
           //selectBtnActive = false;
        }
        else
        {
           //nothing
        }

     }
     else //Button released
     {
        #if _DEBUG_MODE_IS
        Serial.println("end");
        #endif
        selectBtnActive = false;
        ldrElapsedTime = millis() - ldrTimeStamp;
        performAction();
     }
  }
  else
  {
     if(selectBtn.Read())
     {
        ldrTimeStamp = millis();
        selectBtnActive = true;
        #if _DEBUG_MODE_IS
        Serial.println("Start");
        #endif
     }
  }
  // Go to stand by for inactivity
  if(CurrentPowerState() == ON_STATE)
   {
      if((lastActionTimeStamp > (millis() + TIME_FOR_INACTIVITY)))
      {
         ScreenOff();
      }
   }   
}


void performAction(void)
{
   long int currentTime = millis(); 
  //Change profile
   #if _DEBUG_MODE_IS
   Serial.println("Action");
   #endif
   if((CurrentPowerState() == STANDBY_STATE))
   {
      if(ldrElapsedTime > 1500)
      {
         //Deactivate the screen
         ScreenOn();
      }
   }
   else if(CurrentPowerState() == ON_STATE)
   {
      if((ldrElapsedTime > 50) && (ldrElapsedTime < (1500)))
      {
         ReqNextProfile();
      }
      else if(ldrElapsedTime >  1500)
      {
         ScreenOff();
      }
   }
   #if _DEBUG_MODE_IS
   Serial.println(ldrElapsedTime);
   #endif
   lastActionTimeStamp = millis();
}

void ScreenOff(void)
{
   #if _DEBUG_MODE_IS
   Serial.println("turn Off screen");
   #endif
   newPowerState = STANDBY_STATE; 
}

void ScreenOn(void)
{
   #if _DEBUG_MODE_IS
   Serial.println("turn On screen");
   #endif
   newPowerState = ON_STATE;
   
}

void ReqNextProfile(void)
{
      messageSendChar('N'); 
      messageEnd();
     #if _DEBUG_MODE_IS
     Serial.println("Next profile");
     #endif
  
}

void calibrateLdrButton(void)
{
   long int ldrButtonValueSum = 0;
   long int ldrSensorValueSum = 0;
   //long int ldrSensorValueSum = 0;
   //long int ldrSensorValueSum = 0;
   long int timmer = millis() + 3000;
   int i = 0;
   
   // calibrate during the first seconds 
   while (millis() < timmer) {
     ldrButtonValueSum += analogRead(ldrButtonPort);
     ldrSensorValueSum += analogRead(ldrSensorPort);
     i++;
     delay(100);
   }
   // set the values
   selectBtn.SetTriggerLevel((ldrButtonValueSum / i) + 2*TRESHOLD_WIDTH);
   lightSensor.SetTriggerLevel(ldrSensorValueSum / i *2 );
    #if _DEBUG_MODE_IS
    Serial.print("LdrButton is set to: ");
    Serial.println(ldrButtonValueSum / i);
    //Serial.Println("Ldr is set to: %d", levelLdr);
    #endif
    selectBtnActive = false;
}

int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

