

#include "AnalogSwitch.h"
#include "ioConfig.h"

#define LDR_WAIT_TIME 500 //milliseconds 
#define LDR_NEXT_TIME 300 //milliseconds
#define TIME_FOR_INACTIVITY  15000
#define TRESHOLD_WIDTH  10

unsigned int ldrTimeStamp;
unsigned int ldrElapsedTime;
unsigned int lastActionTimeStamp;
unsigned int levelLdrButton;
AnalogSwitch selectBtn(ldrButtonPort, levelLdrButton, TRESHOLD_WIDTH);
boolean selectBtnActive = false;

void ButtonsTask(void)
{
  if(selectBtnActive)
  {
     if(selectBtn.Read())
     {
        //nothing
     }
     else //Button released
     {
        #if _DEBUG_MODE_IS
        Serial.println("end");
        #endif
        selectBtnActive = false;
        ldrElapsedTime = millis() - ldrTimeStamp;
        performAction();
     }
  }
  else
  {
     if(selectBtn.Read())
     {
        ldrTimeStamp = millis();
        selectBtnActive = true;
        #if _DEBUG_MODE_IS
        Serial.println("Start");
        #endif
     }
  }
  // Go to stand by for inactivity
  if(CurrentPowerState() == ON_STATE)
   {
      if((lastActionTimeStamp > (millis() + TIME_FOR_INACTIVITY)))
      {
         ScreenOff();
      }
   }   
}


void performAction(void)
{
   long int currentTime = millis(); 
  //Change profile
   #if _DEBUG_MODE_IS
   Serial.println("Action");
   #endif
   if((CurrentPowerState() == STANDBY_STATE))
   {
      if(ldrElapsedTime > 1500)
      {
         //Deactivate the screen
         ScreenOn();
      }
   }
   else if(CurrentPowerState() == ON_STATE)
   {
      if((ldrElapsedTime > 50) && (ldrElapsedTime < (1500)))
      {
         ReqNextProfile();
      }
      else if(ldrElapsedTime >  1500)
      {
         ScreenOff();
      }
   }
   #if _DEBUG_MODE_IS
   Serial.println(ldrElapsedTime);
   #endif
   lastActionTimeStamp = millis();
}

void ScreenOff(void)
{
   #if _DEBUG_MODE_IS
   Serial.println("turn Off screen");
   #endif
   newPowerState = STANDBY_STATE; 
}

void ScreenOn(void)
{
   #if _DEBUG_MODE_IS
   Serial.println("turn On screen");
   #endif
   newPowerState = ON_STATE;
   
}

void ReqNextProfile(void)
{
      messageSendChar('N'); 
      messageEnd();
     #if _DEBUG_MODE_IS
     Serial.println("Next profile");
     #endif
  
}

void calibrateLdrButton(void)
{
   long int ldrButtonValueSum = 0;
   long int ldrSensorValueSum = 0;
   //long int ldrSensorValueSum = 0;
   //long int ldrSensorValueSum = 0;
   long int timmer = millis() + 3000;
   int i = 0;
   
   // calibrate during the first seconds 
   while (millis() < timmer) {
     ldrButtonValueSum += analogRead(ldrButtonPort);
     ldrSensorValueSum += analogRead(ldrSensorPort);
     i++;
     delay(100);
   }
   // set the values
   selectBtn.SetTriggerLevel((ldrButtonValueSum / i) + 2*TRESHOLD_WIDTH);
   lightSensor.SetTriggerLevel(ldrSensorValueSum / i *2 );
    #if _DEBUG_MODE_IS
    Serial.print("LdrButton is set to: ");
    Serial.println(ldrButtonValueSum / i);
    //Serial.Println("Ldr is set to: %d", levelLdr);
    #endif
    selectBtnActive = false;
}
