/* This library allows to contraol an analog input as a simple button. It also
 * handles a threshold to make it more feasable.
 * Author:
 * Efrain Calderon
 * E- mail: darkcerano@gmial.com
 */
#include "WProgram.h"
#include "AnalogSwitch.h"

/*
 * two-wire constructor.
 * Sets which wires should control the motor.
 */
AnalogSwitch::AnalogSwitch(int aPin, int triggerLevel, int threshold)
{
  // Arduino pin used:
  this->aPin = aPin;
  
  this->triggerLevel = triggerLevel;
  this->threshold = threshold;
  setActive(false);
  this->inverted = false;

}

boolean AnalogSwitch::Read(void)
{
   int data = analogRead(this->aPin);
   if(data > (this->triggerLevel + this->threshold))
   {
      setActive(true);
   }
   else if(data < (this->triggerLevel - this->threshold))
   {
      setActive(false);
   }
   #if 1
   if(this->aPin ==0) {
   Serial.print("[");
   Serial.print(this->triggerLevel - this->threshold);
   Serial.print(", ");
   Serial.print(this->triggerLevel + this->threshold );
   Serial.println("] ");
   Serial.println(data);}
   #endif
   return this->state;
}

void AnalogSwitch::InvertedMode(boolean inverted)
{
   this->inverted = state;
}

void AnalogSwitch::setActive(boolean state)
{
   if(this->inverted)
   {
      this->state = ~ state;

   }
   else
   {
      this->state = state;
   }
}

void AnalogSwitch::SetTriggerLevel(int level)
{
   this->triggerLevel = level;
   setActive(false);
}
